module("luci.i18n", package.seeall)
require("luci.util")
require("lmo")

table   = {}
i18ndir = luci.util.libpath() .. "/i18n/"
loaded  = {}
context = luci.util.threadlocal()
default = "en"
function clear()
	table = {}
end
function load(file, lang, force)
	lang = lang and lang:gsub("_", "-") or ""
	if force or not loaded[lang] or not loaded[lang][file] then
		local f = lmo.open(i18ndir .. file .. "." .. lang .. ".lmo")
		if f then
			if not table[lang] then
				table[lang] = { f }
				setmetatable(table[lang], {
					__index = function(tbl, key)
						for i = 1, #tbl do
							local s = rawget(tbl, i):lookup(key)
							if s then return s end
						end
					end
				})
			else
				table[lang][#table[lang]+1] = f
			end

			loaded[lang] = loaded[lang] or {}
			loaded[lang][file] = true
			return true
		else
			return false
		end
	else
		return true
	end
end
function loadc(file, force)
	load(file, default, force)
	if context.parent then load(file, context.parent, force) end
	return load(file, context.lang, force)
end
function setlanguage(lang)
	context.lang   = lang:gsub("_", "-")
	context.parent = (context.lang:match("^([a-z][a-z])_"))
end
function translate(key)
	return (table[context.lang] and table[context.lang][key])
		or (table[context.parent] and table[context.parent][key])
		or (table[default] and table[default][key])
		or key
end
function translatef(key, ...)
	return tostring(translate(key)):format(...)
end
function string(key)
	return tostring(translate(key))
end
function stringf(key, ...)
	return tostring(translate(key)):format(...)
end
